@@ -173,6 +173,20 @@ describe "Simple Dataset operations" do
 
   it "should fetch a single row correctly" do
     @ds.first.must_equal(:id=>1, :number=>10)
+    @ds.single_record.must_equal(:id=>1, :number=>10)
+    @ds.single_record!.must_equal(:id=>1, :number=>10)
+  end
+  
+  it "should work correctly when returning from each without iterating over the whole result set" do
+    @ds.insert(:number=>20)
+    @ds.order(:id).each{|v| break v}.must_equal(:id=>1, :number=>10)
+    @ds.reverse(:id).each{|v| break v}.must_equal(:id=>2, :number=>20)
+  end
+  
+  it "should fetch a single value correctly" do
+    @ds.get(:id).must_equal 1
+    @ds.select(:id).single_value.must_equal 1
+    @ds.select(:id).single_value!.must_equal 1
   end
   
   it "should have distinct work with limit" do
@@ -239,6 +253,10 @@ describe "Simple Dataset operations" do
   
   it "should fetch correctly with a limit and offset without an order" do
     @ds.limit(2, 1).all.must_equal []
+    @ds.join(:items___i, :id=>:id).select(:items__id___s, :i__id___id2).limit(2, 1).all.must_equal []
+    @ds.join(:items___i, :id=>:id).select(:items__id).limit(2, 1).all.must_equal []
+    @ds.join(:items___i, :id=>:id).select(Sequel.qualify(:items, :id)).limit(2, 1).all.must_equal []
+    @ds.join(:items___i, :id=>:id).select(Sequel.qualify(:items, :id).as(:s)).limit(2, 1).all.must_equal []
   end
 
   it "should be orderable by column number" do
@@ -957,14 +975,19 @@ describe "Sequel::Dataset convenience methods" do
   end
   
   it "#group_rollup should include hierarchy of groupings" do
-    @ds.group_by(:a).group_rollup.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.must_equal [[1, 10, 16], [2, 7, 11], [nil, 17, 27]]
-    @ds.group_by(:a, :b).group_rollup.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.must_equal [[1, 3, 11], [1, 4, 5], [1, nil, 16], [2, 3, 5], [2, 4, 6], [2, nil, 11], [nil, nil, 27]]
+    @ds.group_by(:a).group_rollup.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.map(&:to_i)}.must_equal [[nil, 17, 27], [1, 10, 16], [2, 7, 11]]
+    @ds.group_by(:a, :b).group_rollup.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.map(&:to_i)}.must_equal [[nil, nil, 27], [1, nil, 16], [1, 3, 11], [1, 4, 5], [2, nil, 11], [2, 3, 5], [2, 4, 6]]
   end if DB.dataset.supports_group_rollup?
 
   it "#group_cube should include all combinations of groupings" do
-    @ds.group_by(:a).group_cube.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.must_equal [[1, 10, 16], [2, 7, 11], [nil, 17, 27]]
-    @ds.group_by(:a, :b).group_cube.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.must_equal [[1, 3, 11], [1, 4, 5], [1, nil, 16], [2, 3, 5], [2, 4, 6], [2, nil, 11], [nil, 3, 16], [nil, 4, 11], [nil, nil, 27]]
+    @ds.group_by(:a).group_cube.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.map(&:to_i)}.must_equal [[nil, 17, 27], [1, 10, 16], [2, 7, 11]]
+    @ds.group_by(:a, :b).group_cube.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.map(&:to_i)}.must_equal [[nil, nil, 27], [nil, 3, 16], [nil, 4, 11], [1, nil, 16], [1, 3, 11], [1, 4, 5], [2, nil, 11], [2, 3, 5], [2, 4, 6]]
   end if DB.dataset.supports_group_cube?
+
+  it "#grouping_sets should include sets specified in group" do
+    @ds.group_by(:a, []).grouping_sets.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.map(&:to_i)}.must_equal [[nil, 17, 27], [1, 10, 16], [2, 7, 11]]
+    @ds.group_by([:a, :b], :a, :b, []).grouping_sets.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.map(&:to_i)}.must_equal [[nil, nil, 27], [nil, 3, 16], [nil, 4, 11], [1, nil, 16], [1, 3, 11], [1, 4, 5], [2, nil, 11], [2, 3, 5], [2, 4, 6]]
+  end if DB.dataset.supports_grouping_sets?
 end
 
 describe "Sequel::Dataset convenience methods" do
@@ -1408,6 +1431,7 @@ describe "Sequel::Dataset DSL support" do
   end
   
   it "should work empty arrays with nulls" do
+    @ds = @ds.extension(:empty_array_consider_nulls)
     @ds.insert(nil, nil)
     @ds.filter(:a=>[]).all.must_equal []
     @ds.exclude(:a=>[]).all.must_equal []
@@ -1425,7 +1449,7 @@ describe "Sequel::Dataset DSL support" do
   end
   
   it "should work empty arrays with nulls and the empty_array_ignore_nulls extension" do
-    ds = @ds.extension(:empty_array_ignore_nulls)
+    ds = @ds
     ds.insert(nil, nil)
     ds.filter(:a=>[]).all.must_equal []
     ds.exclude(:a=>[]).all.must_equal [{:a=>nil, :b=>nil}]
