@@ -92,17 +92,29 @@ describe Sequel::Model, "dataset" do
     proc {@b.dataset}.must_raise(Sequel::Error)
   end
   
-  it "should disregard namespaces for the table name" do
-    begin
-      module ::BlahBlah
-        class MwaHaHa < Sequel::Model
-        end
-      end
-    
-      BlahBlah::MwaHaHa.dataset.sql.must_equal 'SELECT * FROM mwa_ha_has'
-    ensure
-      Object.send(:remove_const, :BlahBlah)
+  it "should not override dataset explicitly set when subclassing" do
+    sc = Class.new(::Elephant) do
+      set_dataset :foo
     end
+    sc.table_name.must_equal :foo
+  end
+end
+  
+describe Sequel::Model, "implicit table names" do
+  after do
+    Object.send(:remove_const, :BlahBlah)
+  end
+  it "should disregard namespaces for the table name" do
+    module ::BlahBlah
+      class MwaHaHa < Sequel::Model
+      end
+    end
+    BlahBlah::MwaHaHa.dataset.sql.must_equal 'SELECT * FROM mwa_ha_has'
+  end
+
+  it "should automatically set datasets when anonymous class of Sequel::Model is used as superclass" do
+    class BlahBlah < Class.new(Sequel::Model); end
+    BlahBlah.dataset.sql.must_equal 'SELECT * FROM blah_blahs'
   end
 end
 
